*-	Program....:	UNIQFILE.PRG
*-	Author.....:	Greg Dunn
*-	Project....:	TIPS - Total Integrated Proposal System
*-	Project....:	TRW FoxTeam Reusable Library
*-	Created....:	10/29/93
*-	Copyright..:	(c) TRW, 1993
*-	Descript...:	Returns a unique file name, given various specifications
*-	Call Samp..:	
*
*		jcTablName = UNIQFILE()
*		jcTablName = UNIQFILE("DBF")
*		jcTablName = LEFT(UNIQFILE("DBF"), 8)  && Yields convenient 8-char alias
*		jcTablName = UNIQFILE("DBF", "TEMP")
*		jcTablName = UNIQFILE("BAK*", "ASSYLIST")
*
*-	Parm List..:	tcExt		-	a 3-character file extension
*							(optionally, a "*" may be appended to trigger
*							iterating on the file extension with a sequential
*							number)
*						tcPfx	-	a 1- to 8-character file prefix
*							(if 8 characters, it's an entire file name,
*							and can only be used with the "XXX*" extension)
*							since otherwise the procedure will have no scope
*							for creating uniqueness.
*-	Needed.....:	Allow sending of a complete file path, not just a
*						file name <= 8 characters, so can guarantee a unique
*						file in a specific, remote directory. 10/29/93
*-	Chg list...:
* ------------------------------------------------------------ *
PARAMETERS tcPfx, tcExt
PRIVATE    tcPfx, tcExt, lnParms, lcUniqName, lcMsg, llExtSeq, lnCtr

lnParms = PARAMETERS()

* Determine which usable parameters were passed
DO CASE
CASE lnParms = 0
	llPfx = .F.
	llExt = .F.
CASE lnParms = 1
	llPfx = IIF( NOT EMPTY(tcPfx), .T., .F.)
	llExt = .F.
CASE lnParms = 2
	llPfx = IIF( NOT EMPTY(tcPfx), .T., .F.)
	llExt = IIF( NOT EMPTY(tcExt), .T., .F.)
CASE lnParms > 2
	lcMsg = "Here in UNIQFILE.  Too many parameters passed from " + ;
		PROGRAM() + "."
	WAIT WINDOW lcMsg
	RETURN .F.
ENDCASE

llExtSeq = .F.
IF NOT llExt
	* Set default extension
	tcExt = "$$$"
ELSE
	*-- Process extension
	tcExt = UPPER(ALLT(tcExt))

	IF NOT BETWEEN(LEN(tcExt), 1, 3)
		IF LEN(tcExt) = 4 AND RIGHT(tcExt,1) = "*"
			* User passed wild card; iterate on extension using sequential
			* ordinal integers
			llExtSeq = .T.
			tcExt = LEFT(tcExt,3)
		ELSE
			lcMsg ="Here in UNIQFILE.  Extension " + tcExt + " is invalid."
			WAIT WINDOW lcMsg
			RETURN .F.
		ENDIF
	ENDIF

	*-- Weed out illegal characters
	IF NOT NameOkay(tcExt)
		RETURN .F.
	ENDIF

ENDIF

IF NOT llPfx
	* Set default prefix
	tcPfx = ""
ELSE
	*-- Process prefix
	tcPfx = UPPER(ALLT(tcPfx))

	DO CASE
	CASE LEN(tcPfx) = 8
		* If user passed a full-length file prefix, he must also
		* have specified a variable extension!
		IF NOT llExtSeq
			lcMsg = "Cannot pass 8-character prefix without " + ;
				"specifying a variable extension. [UNIQFILE()]"
			WAIT WINDOW lcMsg
			RETURN .F.
		ENDIF
	CASE LEN(tcPfx) > 8
		lcMsg = "You may not pass a prefix > 8 characters in length. [UNIQFILE()]"
		WAIT WINDOW lcMsg
		RETURN .F.
	ENDCASE

	*-- Weed out illegal characters
	IF NOT NameOkay(tcPfx)
		RETURN .F.
	ENDIF

ENDIF

lcUniq1st = tcPfx + RIGHT( SYS(3), 8-LEN(tcPfx) )
lcUniqName = lcUniq1st + "." + tcExt
lnCtr = 1

DO WHILE FILE(lcUniqName)
	IF NOT llExtSeq
		lcUniqName = tcPfx + RIGHT(SYS(3), 8 - LEN(tcPfx) ) + "." + tcExt
	ELSE
		* Iterate on extension using sequential numbers
		lnCtr = lnCtr + 1
		DO CASE
		CASE lnCtr < 10
			lcUniqName = lcUniq1st + "." + ALLT(LEFT(tcExt,2)) + TSTR(lnCtr)
		CASE lnCtr < 100
			lcUniqName = lcUniq1st + "." + ALLT(LEFT(tcExt,1)) + TSTR(lnCtr)
		CASE lnCtr < 1000
			lcUniqName = lcUniq1st + "." + TSTR(lnCtr)
		OTHERWISE
			lcMsg = "Too many files with this name!"
			WAIT WINDOW lcMsg
			RETURN .F.
		ENDCASE
	ENDIF
ENDDO
RETURN lcUniqName


* ------------------------------------------------------------ *
PROCEDURE NameOkay
* ------------------------------------------------------------ *
* Checks string for character illegal in DOS file name
* ------------------------------------------------------------ *
PARAMETERS tcStr
PRIVATE tcStr, lcMsg, lnCtr

FOR lnCtr = 1 to LEN(tcStr)
	IF SUBS(tcStr,lnCtr) $ [*/+|><":;?,]
		lcMsg ="Here in UNIQFILE.  Expression " + tcStr + " is invalid."
		=Wwn(lcMsg)
		RETURN .F.
	ENDIF
ENDFOR
RETURN .T.

