* ------------------------------------------------------------ *
*  Program...........: DESNIPIT
*  Author............: Greg Dunn
*  Project...........: 
*  Created...........: 08/18/93
*  Copyright.........: (c) TRW, 1993
*  Description.......: 
*  Calling Samples...: 
*  Parameter List....: 
*  Major change list.: 
* ------------------------------------------------------------ *

set talk off

public glUpdateTm
private lcViewWas
lcViewWas = uniqfile()
create view (lcViewWas) from environment


* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *
do DESNIPIT.SPR
* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *

release glUpdateTm
close data
set view to (lcViewWas)
erase (lcViewWas)
clear
return .t.



* ------------------------------------------------------------ *
procedure BackScr
* ------------------------------------------------------------ *
* Handles backup of the screen tables being converted...
* ------------------------------------------------------------ *
lcOutScx = strtran(lcObj, ".SCX", ".S_X")

*-- Ask for backup file name only if processing a single screen
if lnOneMany = 1
	lcOutScx = putfile("Backup file name", lcOutScx)
endif lnOneMany

if !empty(lcOutScx)
	copy file (lcObj) to (lcOutScx)

	* Copy SCT file same name as SCX, except substitute
	* "T" for "X" in the file extension.  (Remember, the
	* user may have changed the default file extension
	* as well as the default file name.)
	
	lcOutXNam = left(lcOutScx, at(".", lcOutScx) - 1 )
	lcOutXExt = subs(lcOutScx, at(".", lcOutScx) )
	lcOutTExt = strtran(lcOutXExt, "X", "T")
	lcCurSCT  = strtran(lcObj, ".SCX", ".SCT")
	lcOutSCT  = lcOutXNam + lcOutTExt
	copy file (lcCurSCT) to (lcOutSCT)
	return .t.
else
	lcMsg = "No backup file destination specified.  Aborting..."
	=Wwn(lcMsg)
	return .f.
endif


* ------------------------------------------------------------ *
procedure Exterior
* ------------------------------------------------------------ *
* "Exteriorize" a screen:  that is, move all code out of the
* screen's snippets into a PRG that calls the now codeless screen.
* See Brachman, Michael, FoxPro Advisor, Mar 93, p.28 for
* format and rationale.
*
* Uses the following variables set in the corresponding screen:
*
*	lnVersion  =>	determines which .SCX records to process
*					1=Windows only   2=DOS only
*	lnUpdatScx =>	determines whether screen will be changed
*					(i.e., are we copying the code from snippets
*					for inspection, or moving it out for real)
*					1=don't change .SCX     2=do change .SCX
* ------------------------------------------------------------ *
parameters lcObj
private    lcObj, lcScrName, lcDestFile

if lnUpdatScx = 2  && screen will be updated
	* Back up the screen table if user desires...
	lcMsg = "Back up screen table?"
	if lnOneMany = 2 OR YesNo(lcMsg)
		if !BackScr()
			return .f.
		endif
	endif
endif lnUpdatScx = 2  && update the screen

if !xuse(lcObj, "excl alias TargScrn")
	lcMsg = "Unable to open screen table."
	=Wwn(lcMsg)
	return .f.
endif

* Create cursor with memo where output PRG will be built...

create cursor temp (cKey c(12), mScratch m(10))

lcScrName = left(lcObj, rat(".", lcObj) - 1 ) && strip extension
lcNickName = subs(lcScrName, rat("\", lcScrName) + 1)  && isolate short name

* Prompt for output file destination only if processing
* single screen, not multiple ones...
if lnOneMany = 1
	lcDestFile = putfile("Output Proc File:", lcNickName + ".PRG", "PRG")
else
	lcDestFile = lcScrName + ".PRG"
endif

if empty(lcDestFile)
	return .f.
endif

lcCrLf = chr(13) + chr(10)
lcDivider1 = "* " + repl("-", 60) + " *" + lcCrLf
lcDivider2 = "* " + repl("%", 60) + " *" + lcCrLf

*-- We'll assign cKey codes to the snippets that will permit
*-- us to write them out in the following order:
*-- 1.	Proc header
*-- 2.	Setup code (if any)
*--	3.	Call to SPR
*--	4.	Cleanup code (if any)
*--	5.	GET object snippets in alphabetical order
*--	6.	Screen snippets in alphabetical order

* Create procedure header in scratch memo...

*{ Commented at 13:59:49 on October 19, 1993

* lcAuthor = "Greg Dunn"
* lcCopyWr = "(c) TRW, 1993"
* lcProject = ""


lcAuthor = GETENV("author")
lcCopyWr = "(c) TRW, " + tStr(YEAR(DATE()))
lcProject = GETENV("project")

*} End Commenting 13:59:49 - October 19, 1993* You may wish to set up environment variables you can retrieve
* via GETENV() for lcAuthor and lcProject

*{ Commented at 13:55:53 on October 19, 1993

* insert into temp (cKey, mScratch) values					;
* 	(																	;
* 	"01ProcHdr",													;
* 	lcDivider1 + 													;
* 	"*  Program...........: " + lcNickName + lcCrLf	+	;
* 	"*  Author............: " + lcAuthor + lcCrLf	+	;
* 	"*  Project...........: " + lcProject + lcCrLf	+	;
* 	"*  Created...........: " + DTOC(Date()) + lcCrLf +;
* 	"*  Copyright.........: " + lcCopyWr + lcCrLf	+	;
* 	"*  Description.......: " + lcCrLf +					;
* 	"*  Calling Samples...: " + lcCrLf +					;
* 	"*  Parameter List....: " + lcCrLf +					;
* 	"*  Major change list.: " + lcCrLf +					;
* 	+ lcDivider1 + lcCrLf										;
* 	)

insert into temp (cKey, mScratch) values					;
	(																	;
	"01ProcHdr",													;
	lcDivider1 + 													;
	"*-"+chr(9)+"Program....:"+chr(9) + lcNickName + lcCrLf	+	;
	"*-"+chr(9)+"Author.....:"+chr(9) + lcAuthor + lcCrLf	+		;
	"*-"+chr(9)+"Project....:"+chr(9) + lcProject + lcCrLf	+	;
	"*-"+chr(9)+"Created....:"+chr(9) + DTOC(Date()) + lcCrLf +	;
	"*-"+chr(9)+"Copyright..:"+chr(9) + lcCopyWr + lcCrLf	+		;
	"*-"+chr(9)+"Descript...:"+chr(9) + lcCrLf +	;
	"*-"+chr(9)+"Call Samp..:"+chr(9) + lcCrLf +	;
	"*-"+chr(9)+"Parm List..:"+chr(9) + lcCrLf +	;
	"*-"+chr(9)+"Chg list...:"+chr(9) + lcCrLf +	;
	+ lcDivider1 + lcCrLf								;
	)

*} End Commenting 13:55:53 - October 19, 1993
sele TargScrn

* Limit records processed to those for version specified...
do case
case lnVersion = 1  && Windows
	set filter to TargScrn.PlatForm = "WINDOWS"
case lnVersion = 2  && DOS
	set filter to TargScrn.PlatForm = "DOS"
otherwise
	* shouldn't occur yet
endcase

go top   && first record has SETUP, CLEANUP, and READ snippets...

glUpdateTm = .f.   && whether to update main screen record's time stamp


*-- Write SETUP code
if TargScrn.setuptype > 0 and !empty(TargScrn.setupcode)
	* There is code in the screen's SETUP snippet...
	=WriteSnp(lcNickName, "SETUPCODE", "screen")
endif

* Write out the call to the screen SPR...
insert into temp (cKey, mScratch) values	;
	(										;
		"03Call2Scr",						;
		lcDivider2	+						;
		"do " + lcNickName + ".SPR" +		;
		lcCrLf + 							;
		lcDivider2	+						;
		lcCrLf								;
	)

*-- Write CLEANUP code
if TargScrn.proctype > 0 and !empty(TargScrn.proccode)
	* There is code in the screen's CLEANUP snippet...
	=WriteSnp(lcNickName, "PROCCODE", "screen")
endif

*-- Process the screen's READ snippets.
go top
if TargScrn.whentype > 0 and !empty(TargScrn.when)
	* There is code in the screen's (READ) WHEN snippet...
	=WriteSnp(lcNickName, "WHEN", "screen")
endif

if TargScrn.validtype > 0 and !empty(TargScrn.valid)
	* There is code in the screen's (READ) VALID snippet...
	=WriteSnp(lcNickName, "VALID", "screen")
endif

if TargScrn.showtype > 0 and !empty(TargScrn.show)
	* There is code in the screen's (READ) SHOW snippet...
	=WriteSnp(lcNickName, "SHOW", "screen")
endif

if TargScrn.activtype > 0 and !empty(TargScrn.activate)
	* There is code in the screen's (READ) ACTIVATE snippet...
	=WriteSnp(lcNickName, "ACTIVATE", "screen")
endif

if TargScrn.deacttype > 0 and !empty(TargScrn.deactivate)
	* There is code in the screen's (READ) DEACTIVATE snippet...
	=WriteSnp(lcNickName, "DEACTIVATE", "screen")
endif

if glUpdateTm
	* Update record's timestamp so Fox will regenerate the code...
	repl TargScrn.timestamp with TargScrn.timestamp + 1
endif

*-- Process the GET records...

skip  && They begin with the 2nd record of the table
scan rest

	glUpdateTm = .f.  && Reset for each new record processed
	
	if TargScrn.errortype > 0 and !empty(TargScrn.error)
		* There is code in the GET object's ERROR snippet...
		=WriteSnp(lcNickName, "ERROR", "GET object")
	endif

	if TargScrn.hitype > 0 and !empty(TargScrn.rangehi)
		* There is code in the GET object's RANGE UPPER snippet...
		=WriteSnp(lcNickName, "RANGEHI", "GET object")
	endif

	if TargScrn.lotype > 0 and !empty(TargScrn.rangelo)
		* There is code in the GET object's RANGE LOWER snippet...
		=WriteSnp(lcNickName, "RANGELO", "GET object")
	endif

	if TargScrn.messtype > 0 and !empty(TargScrn.message)
		* There is code in the GET object's MESSAGE snippet...
		=WriteSnp(lcNickName, "MESSAGE", "GET object")
	endif

	if TargScrn.validtype > 0 and !empty(TargScrn.valid)
		* There is code in the GET object's VALID snippet...
		=WriteSnp(lcNickName, "VALID", "GET object")
	endif

	if TargScrn.whentype > 0 and !empty(TargScrn.when)
		* There is code in the GET object's WHEN snippet...
		=WriteSnp(lcNickName, "WHEN", "GET object")
	endif

	if glUpdateTm
		* Update record's timestamp...
		repl TargScrn.timestamp with TargScrn.timestamp + 1
	endif

endscan

*-- Concatenate the pieces, then write the PRG...
create cursor temp2 (mScratch m(10))
appe blank

sele temp
index on cKey tag cKey
go top
scan
	repl temp2.mScratch with temp.mScratch additive
endscan
copy memo temp2.mScratch to (lcDestFile)

if !wexist("wbDestFile")
	define window wbDestFile	;
		from 0,0 to srows()-1, scols()-1	;
		float grow minimize zoom close system	;
		color scheme 8	;
		title "DONE! >> " + lcDestFile + " <<"
endif

if lnOneMany = 1
	modify file (lcDestFile) window wbDestFile
endif

release window wbDestFile
close data

return .t.



* ------------------------------------------------------------ *
procedure MlhAction
* ------------------------------------------------------------ *
* Message snippet for GET object lhAction
* ------------------------------------------------------------ *
return "Proceed with, or cancel, selection of screens and desnippetization."



* ------------------------------------------------------------ *
procedure MlnOneMany
* ------------------------------------------------------------ *
* Message snippet for GET object lnOneMany
* ------------------------------------------------------------ *
return "Desnippetize one screen, or a directory full of them."



* ------------------------------------------------------------ *
procedure MlnUpdatSc
* ------------------------------------------------------------ *
* Message snippet for GET object lnUpdatSc
* ------------------------------------------------------------ *
return "COPY the code for inspection; MOVE to desnippetize the screen."



* ------------------------------------------------------------ *
procedure MlnVersion
* ------------------------------------------------------------ *
* Message snippet for GET object lnVersion
* ------------------------------------------------------------ *
return "DOS and Windows snippets are stored separately."



* ------------------------------------------------------------ *
procedure ManyScx
* ------------------------------------------------------------ *
* Processing loop for all screens in a directory
* ------------------------------------------------------------ *
private lcObj

lcDir = getdir('', 'Directory with screens:')
if empty(lcDir)
	return .t.
endif

lnScreens=adir(laScreen,lcDir + "*.SCX")
if _tally = 0
	lcMsg = "No screens to process."
	=Wwn(lcMsg)
	return .t.
endif

for lnCtr = 1 to lnScreens
	lcObj = lcDir + ALLT(laScreen[lnCtr, 1])
	lcMsg = "Desnippetizing screen " + lcObj
	=Wwn(lcMsg)
	=Exterior(lcObj)
endfor

lcMsg = "All screens processed."
=Wwn(lcMsg)

return .t.



* ------------------------------------------------------------ *
procedure OneScx
* ------------------------------------------------------------ *
* Processes single screen
* ------------------------------------------------------------ *
private lcObj

lcObj = getfile('SCX', 'Screen to process:', 'Select')
if !empty(lcObj)
	=Exterior(lcObj)
else
	return .t.
endif

return .t.


* ------------------------------------------------------------ *
procedure SnPrefix
* ------------------------------------------------------------ *
* Returns 1-character standard prefix for GET object snippets,
* based on snippet type
*
* Parameters:
*		lcWhichSnp	-	Snippet type
*		{""WHEN", "VALID", "RANGELO", "RANGEHI", "ERROR", "MESSAGE"}
* Sample calls:
*		lcPreFix=SnPrefix(lcWhichSnp)
*
* ------------------------------------------------------------ *

parameters lcWhichSnp
private    lcWhichSnp, lcPreFix

do case
case lcWhichSnp = "WHEN"
	lcPreFix = "W"
case lcWhichSnp = "VALID"
	lcPreFix = "V"
case lcWhichSnp = "RANGELO"
	lcPreFix = "L"
case lcWhichSnp = "RANGEHI"
	lcPreFix = "H"
case lcWhichSnp = "ERROR"
	lcPreFix = "E"
case lcWhichSnp = "MESSAGE"
	lcPreFix = "M"
otherwise  && in case something weird got passed
	lcPreFix = "^@"
endcase

return lcPreFix


* ------------------------------------------------------------ *
Procedure SnTypFld
* ------------------------------------------------------------ *
* Returns name of "type" field from .SCX table corresponding 
* to a specified snippet name
*
* Parameters:
*		lcWhichSnp	-	Snippet type
*						{"SETUPCODE", "PROCCODE", "WHEN", "VALID", 
*						 "ACTIVATE", "DEACTIVATE",
*						 "RANGELO", "RANGEHI", 
*						 "ERROR", "MESSAGE"}
* Sample calls:
*		=SnTypFld("ACTIVATE")
*
* Returns:
*	'Type' field corresponding to snippet name
*	{"SETUPTYPE", "PROCTYPE", "WHENTYPE",
*	 "VALIDTYPE", "ACTIVTYPE", "DEACTTYPE",
*	 "LOTYPE", "HITYPE", "ERRORTYPE", "MESSTYPE"}

* ------------------------------------------------------------ *

parameters lcWhichSnp
private    lcWhichSnp

do case
case lcWhichSnp = "SETUPCODE"
	lcTypFld = "SETUPTYPE"
case lcWhichSnp = "PROCCODE"
	lcTypFld = "PROCTYPE"
case lcWhichSnp = "WHEN"
	lcTypFld = "WHENTYPE"
case lcWhichSnp = "VALID"
	lcTypFld = "VALIDTYPE"
case lcWhichSnp = "ACTIVATE"
	lcTypFld = "ACTIVTYPE"
case lcWhichSnp = "DEACTIVATE"
	lcTypFld = "DEACTTYPE"
case lcWhichSnp = "SHOW"
	lcTypFld = "SHOWTYPE"
case lcWhichSnp = "RANGELO"
	lcTypFld = "LOTYPE"
case lcWhichSnp = "RANGEHI"
	lcTypFld = "HITYPE"
case lcWhichSnp = "ERROR"
	lcTypFld = "ERRORTYPE"
case lcWhichSnp = "MESSAGE"
	lcTypFld = "MESSTYPE"
otherwise
	lcMsg = "Error!  Unknown snippet field: " + lcWhichSnp
	wait window lcMsg timeout 3
endcase

return lcTypFld


* ------------------------------------------------------------ *
procedure VlhAction
* ------------------------------------------------------------ *
* Valid for pushbutton lhAction
* ------------------------------------------------------------ *
if lhAction = 2
	clear read
	return
endif

if lnOneMany = 1  && process single screen
	do OneScx
else              && process many screens
	do ManyScx
endif

RETURN


* ------------------------------------------------------------ *
procedure WriteSnp
* ------------------------------------------------------------ *
* Writes a snippet to the target memo field.
* Optionally, also removes snippet code from the screen being 
* transformed and updates other .SCX information according.
*
* Parameters:
*		tcScrName	-	Name of object (8 chars or less)
*		tcWhichSnp	-	Snippet type
*						{"SETUPCODE", "PROCCODE", "WHEN", "VALID", 
*						 "ACTIVATE", "DEACTIVATE",
*						 "RANGELO", "RANGEHI", 
*						 "ERROR", "MESSAGE"}
*		tcObjType	-	{"screen", "GET object"}
*
* Sample calls:
*		=WriteSnp(lcNickName, "SETUP", "screen")
*		=WriteSnp(lcNickName, "VALID", "GET object")
*
* ------------------------------------------------------------ *

parameters tcScrName, tcWhichSnp, tcObjType
private    tcScrName, tcWhichSnp, tcObjType
private lcFuncName, lcObjName, lcSrcField, lcMsg

if !tcWhichSnp $ "{SETUPCODE, PROCCODE, WHEN, VALID, ACTIVATE}" +	;
	"{DEACTIVATE, SHOW, RANGELO, RANGEHI, ERROR, MESSAGE}"
	lcMsg = "Unknown snippet field: " + tcWhichSnp
	wait window lcMsg timeout 3
	return .f.
endif

do case
case upper(tcObjType) = "SCREEN"
	* If it's a screen snippet (i.e., a READ clause), it is
	* given a prefix of "z" and a name describing its function.
	* The "z" is used so all such screen snippets will sort together
	* at the bottom of the proc code.
 	lcObjName  = tcScrName
 	lcFuncName = "z" + proper(left(tcWhichSnp,4))

case upper(tcObjType) = "GET"
	* If it's a GET snippet, it is named for the GET (up to 9
	* characters), and prefixed by an upper case character
	* indicating function.
	
	* Strip off any "m."...
	lcObjName = subs(TargScrn.name, at(".", TargScrn.name) + 1, 9 )
	lcFuncName = SnPrefix(tcWhichSnp) + lcObjName
endcase


* Write headers for all procedures, but not for the setup or 
* cleanup code...

insert into temp (cKey, mScratch) values ("", "")

do case
case tcWhichSnp = "SETUPCODE"
	repl temp.cKey with "02Setup"
case tcWhichSnp = "PROCCODE"
	repl temp.cKey with "04Cleanup"
otherwise
	repl temp.cKey with "05" + lcFuncName,					;
		temp.mScratch with 									;
			lcDivider1 + 									;
			"procedure " + lcFuncName + lcCrLf +			;
			lcDivider1 + 									;
			"* " + upper(tcWhichSnp) + " snippet for " +	;
			tcObjType +	" " + lcObjName + lcCrLf +			;
			lcDivider1 + lcCrLf
endcase

* Now write the snippet code to the scratch memo field...

lcSrcField = "TargScrn." + tcWhichSnp

*-- Stuff the snippet into the memo field where we're building the PRG.

if tcWhichSnp = "SETUPCODE"
	* If processing the SETUP code, we must comment out screen generator
	* directives

	repl temp.mScratch with ;
		strtran(lcCrLf + &lcSrcField., lcCrLf + "#", lcCrLf + "*--> #") + ;
		lcCrLf + lcCrLf + lcCrLf additive
	* We're adding a CrLf at the beginning of the text, then prefixing
	* all #'s preceded by a CrLf -- that is, all #'s at the start of a
	* line -- with a "*" to make them comments
else
	repl temp.mScratch with ;
		&lcSrcField + lcCrLf + lcCrLf + lcCrLf additive
endif

* If screen tables are to be modified:

if lnUpdatScx = 2  && update the screen
	* For all snippets, we will need to replace the "type" field 
	* in the source .SCX with the value expected for the state in 
	* which we'll leave things.  Get the name of the 'type' field
	* for the current snippet...

	lcTypeFld = SnTypFld(tcWhichSnp)

	* If snippet just written was SETUP or CLEANUP (PROCCODE), simply
	* clear out the source field.  If any other snippet, replace the 
	* memo field in the source .SCX with a call to the newly
	* created function...

	if tcWhichSnp $ "{SETUPCODE, PROCCODE}"
		repl &lcSrcField with ""
		repl &lcTypeFld with 1   && Type number for 'no snippet'
	else
		repl &lcSrcField with lcFuncName + "()"
		repl &lcTypeFld with 0   && Type number for an expression
	endif

	* If this procedure has been called, the current record needs its
	* TIMESTAMP updated...
	glUpdateTm = .t.
endif lnUpdatScx = 2  && update the screen

return .t.


* ------------------------------------------------------------ *
procedure zWhen
* ------------------------------------------------------------ *
* WHEN snippet for MoveSnip.Scx
* ------------------------------------------------------------ *
show get lnScopeOp,2 disable

return .t.



